import argparse
import csv
import json
import logging
import os
import sys
from collections.abc import Iterator
from contextlib import contextmanager

import sqlalchemy as sa
from rdflib import Graph
from rdflib.util import SUFFIX_FORMAT_MAP
from rdflib.util import guess_format
from skosprovider.providers import DictionaryProvider
from skosprovider.providers import SimpleCsvProvider
from skosprovider.providers import VocabularyProvider
from skosprovider.skos import ConceptScheme
from skosprovider.uri import UriPatternGenerator
from skosprovider_rdf.providers import RDFProvider
from skosprovider_sqlalchemy.models import ConceptScheme as ConceptschemeDb
from skosprovider_sqlalchemy.utils import import_provider
from sqlalchemy import create_engine
from sqlalchemy.engine import url
from sqlalchemy.exc import DatabaseError
from sqlalchemy.orm import Session

from atramhasis.data.models import Provider
from atramhasis.scripts.migrate_sqlalchemy_providers import json_serial


def file_to_rdf_provider(uri_pattern: str | None = None, **kwargs) -> RDFProvider:
    """
    Create RDF provider from the input file
    """
    input_file = kwargs.get('input_file')
    _, input_ext = os.path.splitext(os.path.basename(input_file))
    meta_id = kwargs.get('provider_id')
    graph = Graph()
    graph.parse(input_file, format=guess_format(input_ext))
    provider_kwargs = {}
    if uri_pattern:
        uri_generator = UriPatternGenerator(uri_pattern)
        provider_kwargs['uri_generator'] = uri_generator
    return RDFProvider({'id': meta_id}, graph, **provider_kwargs)


def _create_provider_kwargs(**kwargs):
    provider_kwargs = {}
    uri_pattern = kwargs.get('uri_pattern')
    if uri_pattern:
        provider_kwargs['uri_generator'] = UriPatternGenerator(uri_pattern)
    concept_scheme = kwargs.get('concept_scheme')
    if concept_scheme:
        provider_kwargs['concept_scheme'] = concept_scheme
    return provider_kwargs


def file_to_csv_provider(**kwargs) -> SimpleCsvProvider:
    """
    Create CSV provider from the input file
    """
    input_file = kwargs.get('input_file')
    meta_id = kwargs.get('provider_id')
    provider_kwargs = _create_provider_kwargs(**kwargs)
    with open(input_file) as ifile:
        reader = csv.reader(ifile)
        return SimpleCsvProvider({'id': meta_id}, reader, **provider_kwargs)


def file_to_json_provider(**kwargs) -> DictionaryProvider:
    """
    Create Dictionary provider from the input file
    """
    input_file = kwargs.get('input_file')
    meta_id = kwargs.get('provider_id')
    provider_kwargs = _create_provider_kwargs(**kwargs)
    with open(input_file) as data_file:
        dictionary = json.load(data_file)
    return DictionaryProvider({'id': meta_id}, dictionary, **provider_kwargs)


supported_types = {
    'JSON': {'extensions': ['.json'], 'file_to_provider': file_to_json_provider},
    'RDF': {
        'extensions': ['.%s' % suffix for suffix in SUFFIX_FORMAT_MAP],
        'file_to_provider': file_to_rdf_provider,
    },
    'CSV': {'extensions': ['.csv'], 'file_to_provider': file_to_csv_provider},
}

supported_ext = [
    item
    for sublist in [
        supported_types[filetype]['extensions'] for filetype in supported_types.keys()
    ]
    for item in sublist
]


def parse_argv_for_import(argv):
    """
    Parse parameters and validate
    """
    cmd = os.path.basename(argv[0])
    parser = argparse.ArgumentParser(
        description='Import file to a database',
        epilog=(
            f'example: {cmd} '
            'atramhasis/scripts/my_file '
            '--uri-pattern urn:x-skosprovider:trees:%s '
            '--to sqlite:///atramhasis.sqlite '
            '--conceptscheme-label Labels '
            '--conceptscheme-uri urn:x-skosprovider:trees '
            '--create-provider '
            '--provider-id ERFGOEDTYPES '
            '--id-generation-strategy numeric'
        ),
    )
    parser.add_argument(
        'input_file',
        type=str,
        help='local path to the input file',
    )
    parser.add_argument(
        '--uri-pattern',
        dest='uri_pattern',
        type=str,
        help='URI pattern input for the URIGenerator',
    )
    parser.add_argument(
        '--to',
        dest='to',
        metavar='conn_string',
        type=str,
        help='Connection string of the output database',
        required=False,
        default='sqlite:///atramhasis.sqlite',
    )
    parser.add_argument(
        '--conceptscheme-label',
        dest='cs_label',
        type=str,
        help='Label of the conceptscheme',
        required=False,
        default=None,
    )
    parser.add_argument(
        '--conceptscheme-uri',
        dest='cs_uri',
        type=str,
        help='URI of the conceptscheme',
        required=False,
        default=None,
    )
    parser.add_argument(
        '--create-provider',
        dest='create_provider',
        default=True,
        action=argparse.BooleanOptionalAction,
        help='An optional parameter if given a provider is created. '
        'Use --no-create-provider to not create a provider',
    )
    parser.add_argument(
        '--provider-id',
        dest='provider_id',
        type=str,
        help='An optional string (eg. ERFGOEDTYPES) to be assigned to the provider id. '
        'If not specified, assign the conceptscheme id to the provider id',
        required=False,
        default=None,
    )
    parser.add_argument(
        '--id-generation-strategy',
        dest='id_generation_strategy',
        type=str,
        help='URI pattern input for the URIGenerator',
        required=False,
        choices=['numeric', 'guid', 'manual'],
        default=None,
    )
    args = parser.parse_args()
    if not validate_file(args.input_file) or not validate_connection_string(args.to):
        sys.exit(1)

    if args.create_provider is False and (
        args.uri_pattern or args.provider_id or args.id_generation_strategy
    ):
        print(
            '--uri-pattern, --provider-id and --id-generation-strategy can only be '
            'used when --create-provider is set to True'
        )
        sys.exit(1)
    else:
        validate_uri_pattern(args.uri_pattern)
    if args.id_generation_strategy is None:
        args.id_generation_strategy = 'numeric'
    return args


def validate_uri_pattern(uri_pattern: str):
    if not uri_pattern:
        print(
            '--uri-pattern is required when creating a provider. '
            'This can be disabled with --no-create-provider'
        )
        sys.exit(1)
    if uri_pattern.count('%s') != 1:
        print('--uri-pattern must contain exactly one %s placeholder')
        sys.exit(1)


def validate_file(input_file):
    if not os.path.exists(input_file):
        print(f'The input file {input_file} does not exists')
        return False
    elif os.path.splitext(input_file)[1] not in supported_ext:
        print(
            f'the input file {input_file} is not supported. Allowed extensions are: {supported_ext}'
        )
        return False
    else:
        return True


def validate_connection_string(connection_string):
    """
    Validate the connection string
    :param connection_string
    :return: Boolean True if correct connection string
    """
    u = url.make_url(connection_string)
    if u.drivername == 'postgresql':
        if u.username and u.password and u.host and u.port and u.database:
            return True
    elif u.drivername == 'sqlite':
        if u.database:
            # Extract the file path
            sqlite_path = u.database
            if not os.path.exists(sqlite_path):
                print(f'SQLite database file does not exist: {sqlite_path}')
                return False
            return True
    elif u.drivername:
        print('The database driver ' + u.drivername + ' is not supported.')
    print('Wrong structure of connection string "' + connection_string + '"')
    print(
        'Structure: postgresql://username:password@host:port/db_name OR sqlite:///path/db_name.sqlite'
    )
    return False


@contextmanager
def db_session(connection_string: str) -> Iterator[Session]:
    """
    Create a database session based on the connection string
    :param connection_string: connection string
    :return: session
    """
    print(f'Connecting to database {connection_string} ...')
    engine = create_engine(connection_string)
    try:
        with Session(engine) as session:
            yield session
            session.commit()
    finally:
        engine.dispose()


def create_conceptscheme(
    conceptscheme_uri: str, conceptscheme_label: str
) -> ConceptScheme:
    """
    Create a conceptscheme based on arg values
    """
    return ConceptScheme(uri=conceptscheme_uri, labels=[{'label': conceptscheme_label}])


def main(argv=sys.argv):
    """
    Documentation: import -h
    Run: import
    <path_input_file>
    <uri_pattern>
    --to <conn_string>
    --conceptscheme-uri <cs_uri>
    --conceptscheme-label <cs_label>
    --create-provider
    --provider-id <provider_id>
    --id-generation-strategy <numeric/guid/manual>

    example path_input_file:
     atramhasis/scripts/my_file

    structure conn_string:
     postgresql://username:password@host:port/db_name
     sqlite:///path/db_name.sqlite
    default conn_string:
     sqlite:///atramhasis.sqlite

    example conceptscheme_label
     My Conceptscheme
    default conceptscheme_label is the name of the file if a URI is specified.
    If no URI is specified, a conceptscheme will be imported from the input
    file. This only works for RDf files. For other file types (JSON and CSV)
    conceptscheme_uri is mandatory and conceptscheme_label is recommended.
    """

    # Import the data
    args = parse_argv_for_import(argv)
    print(f'Importing file {args.input_file} to database {args.to} ...')

    setup_logging()

    input_name, input_ext = os.path.splitext(os.path.basename(args.input_file))
    file_to_provider_function = [
        supported_types[filetype]['file_to_provider']
        for filetype in supported_types.keys()
        if input_ext in supported_types[filetype]['extensions']
    ][0]
    if args.cs_uri:
        cs_uri = args.cs_uri
        cs_label = args.cs_label if args.cs_label else input_name.capitalize()
        args.concept_scheme = create_conceptscheme(cs_uri, cs_label)
    provider = file_to_provider_function(**vars(args))

    with db_session(args.to) as session:
        validate_database_not_empty(args, session)
        db_conceptscheme = import_provider(provider, session)
        if args.create_provider:
            if not provider.metadata.get('id'):
                provider.metadata['id'] = db_conceptscheme.id
            create_provider(
                id_generation_strategy=args.id_generation_strategy.upper(),
                provider=provider,
                session=session,
                conceptscheme=db_conceptscheme,
            )

        # Get info to return to the user
        session.flush()
        scheme_id = db_conceptscheme.id
    if not args.create_provider:
        prov_id = getattr(args, 'provider_id', None) or input_name.upper()
        print(
            "\n\n*** The import of conceptscheme {0} from the {1} file to {2} was succesful. ***\
              \n\nTo use the data in Atramhasis, you must edit the file my_thesaurus/skos/__init__.py.\
              \nAdd a configuration similar to:\
                \n\ndef create_registry(request):\
                \n\t# create the SKOS registry\
                \n\tregistry = Registry(instance_scope='threaded_thread')\
                \n\t{3} = SQLAlchemyProvider(\
                \n\t\t{{'id': '{4}', 'conceptscheme_id': {5}}},\
                \n\t\trequest.db\
                \n\t)\
                \n\tregistry.register_provider({6})\
                \n\treturn registry\
                \n\n".format(
                prov_id,
                args.input_file,
                args.to,
                prov_id.replace(' ', '_'),
                prov_id,
                scheme_id,
                prov_id.replace(' ', '_'),
            )
        )
    else:
        prov_id = args.provider_id or scheme_id
        msg = """
***
The import of conceptscheme {0} from the {1} file to {2} was succesful.
You can now continue through the Atramhasis UI.
***
"""
        print(msg.format(prov_id, args.input_file, args.to))


def validate_database_not_empty(args, session):
    query = sa.text('SELECT version_num FROM alembic_version')
    try:
        session.execute(query)
    except DatabaseError:
        print(
            f'The database at {args.to} is not up to date. '
            'Please run the migration script first.'
        )
        sys.exit(1)
    else:
        print(' ∟Database connection healthy')


def setup_logging():
    import_logger = logging.getLogger('skosprovider_sqlalchemy.utils')
    import_logger.setLevel(logging.INFO)
    logging_handler = logging.StreamHandler(sys.stdout)
    logging_handler.setFormatter(logging.Formatter('%(message)s'))
    import_logger.addHandler(logging_handler)


def create_provider(
    id_generation_strategy: str,
    provider: VocabularyProvider,
    session: Session,
    conceptscheme: ConceptschemeDb,
):
    db_provider = Provider()
    provider.metadata['atramhasis.id_generation_strategy'] = id_generation_strategy
    db_provider.meta = json.loads(json.dumps(provider.metadata, default=json_serial))
    db_provider.expand_strategy = 'RECURSE'
    db_provider.conceptscheme = conceptscheme
    db_provider.id = provider.metadata.get('id') or conceptscheme.id
    db_provider.uri_pattern = provider.uri_generator.pattern
    if 'conceptscheme_id' in db_provider.meta:
        del db_provider.meta['conceptscheme_id']
    session.add(db_provider)


if __name__ == '__main__':
    main()



import unittest

from webtest import TestApp

from atramhasis import main
from tests import SETTINGS


class StaticTests(unittest.TestCase):
    app = main({}, **SETTINGS)

    def setUp(self):
        self.testapp = TestApp(self.app)

    def test_sitemap_view(self):
        response = self.testapp.get('/sitemap_index.xml')
        self.assertEqual(200, response.status_code)



import os
import unittest
from unittest import mock
from unittest.mock import Mock

import pytest
from paste.deploy.loadwsgi import appconfig
from pyramid import testing
from pyramid.config.settings import Settings
from pyramid.httpexceptions import HTTPNoContent
from pyramid.request import apply_request_extensions
from pyramid.testing import DummyRequest
from skosprovider.registry import Registry
from skosprovider.uri import DefaultUrnGenerator
from skosprovider_sqlalchemy.models import Collection
from skosprovider_sqlalchemy.models import Concept
from skosprovider_sqlalchemy.models import ConceptScheme
from skosprovider_sqlalchemy.models import Label
from skosprovider_sqlalchemy.models import LabelType
from skosprovider_sqlalchemy.models import Note
from skosprovider_sqlalchemy.models import Thing
from skosprovider_sqlalchemy.providers import SQLAlchemyProvider
from sqlalchemy.exc import NoResultFound
from sqlalchemy.orm import Session
from webob.multidict import MultiDict
from skosprovider.skos import Concept as SkosConcept

import atramhasis
from atramhasis.errors import ConceptNotFoundException
from atramhasis.errors import ConceptSchemeNotFoundException
from atramhasis.errors import SkosRegistryNotFoundException
from atramhasis.errors import ValidationError
from atramhasis.data.models import IDGenerationStrategy
from atramhasis.views.crud import AtramhasisCrud
from atramhasis.views.views import AtramhasisAdminView
from atramhasis.views.views import AtramhasisListView
from atramhasis.views.views import AtramhasisView
from atramhasis.views.views import get_definition
from atramhasis.views.views import get_public_conceptschemes
from atramhasis.views.views import labels_to_string
from fixtures.data import trees

TEST_DIR = os.path.dirname(__file__)
settings = appconfig('config:' + os.path.join(TEST_DIR, 'conf_test.ini'))


def provider(some_id):
    provider_mock = Mock()
    if some_id == 1:
        provider_mock.get_vocabulary_id = Mock(return_value='TREES')
        provider_mock.get_metadata = Mock(return_value={'id': some_id, 'subject': []})
    provider_mock.allowed_instance_scopes = ['single', 'threaded_thread']
    provider_mock.conceptscheme_id = Mock(return_value=some_id)
    provider_mock.metadata = {}
    return provider_mock


def hidden_provider(some_id):
    provider_mock = provider(some_id)
    provider_mock.get_metadata = Mock(
        return_value={'id': some_id, 'subject': ['hidden']}
    )
    return provider_mock


def external_provider(some_id):
    provider_mock = provider(some_id)
    provider_mock.get_metadata = Mock(
        return_value={'id': some_id, 'subject': ['external']}
    )
    return provider_mock


class DummySKOSManager:
    def get_thing(self, concept_id, conceptscheme_id):
        if concept_id == '1':
            return Concept(
                concept_id=concept_id,
                conceptscheme_id=conceptscheme_id,
                labels=[Label(label='test', language_id='any')],
            )
        elif concept_id == '3':
            return Collection(
                concept_id=concept_id,
                conceptscheme_id=conceptscheme_id,
                labels=[Label(label='test', language_id='any')],
            )
        elif concept_id == '555':
            return Thing(concept_id=concept_id, conceptscheme_id=conceptscheme_id)
        elif concept_id == '666':
            raise NoResultFound()

    def get_by_list_type(self, _):
        return [
            LabelType(name='prefLabel', description='foo'),
            LabelType(name='altLabel', description='foo'),
        ]


class DummyConceptschemeManager:
    def get_all(self, conceptscheme_id):
        return [
            Concept(
                concept_id=7895,
                conceptscheme_id=conceptscheme_id,
                type='concept',
                labels=[
                    Label(label='De Paardekastanje', language_id='nl'),
                    Label(label='The Chestnut', language_id='en'),
                    Label(label='la châtaigne', language_id='fr'),
                ],
            ),
            Concept(
                concept_id=9863,
                conceptscheme_id=conceptscheme_id,
                type='concept',
                labels=[Label(label='test', language_id='nl')],
            ),
        ]

    def find(self, conceptscheme_id, _):
        return self.get_all(conceptscheme_id)


class DummyAuditManager:
    def save(self, audit):
        return audit


class TestAtramhasisView(unittest.TestCase):
    def setUp(self):
        self.config = testing.setUp()
        self.request = testing.DummyRequest()
        self.request.data_managers = {
            'skos_manager': None,
            'conceptscheme_manager': None,
            'audit_manager': None,
        }

    def tearDown(self):
        testing.tearDown()

    def test_no_registry(self):
        with self.assertRaises(SkosRegistryNotFoundException):
            AtramhasisView(self.request)


class TestHomeView(unittest.TestCase):
    def setUp(self):
        self.config = testing.setUp()
        self.regis = Registry()
        self.regis.register_provider(trees)
        self.regis.register_provider(hidden_provider(2))
        self.request = testing.DummyRequest()
        self.request.data_managers = {
            'skos_manager': None,
            'conceptscheme_manager': None,
            'audit_manager': None,
        }

    def tearDown(self):
        testing.tearDown()

    def test_passing_view(self):
        self.request.skos_registry = self.regis
        atramhasisview = AtramhasisView(self.request)
        info = atramhasisview.home_view()
        self.assertIsNotNone(info['conceptschemes'][0])
        self.assertEqual(info['conceptschemes'][0]['id'], 'TREES')
        self.assertEqual(1, len(info['conceptschemes']))


class TestFavicoView(unittest.TestCase):
    def setUp(self):
        self.config = testing.setUp()
        self.regis = Registry()
        self.regis.register_provider(trees)
        self.request = testing.DummyRequest()
        self.request.data_managers = {
            'skos_manager': None,
            'conceptscheme_manager': None,
            'audit_manager': None,
        }

    def tearDown(self):
        testing.tearDown()

    def test_passing_view(self):
        self.request.skos_registry = self.regis
        atramhasisview = AtramhasisView(self.request)
        response = atramhasisview.favicon_view()
        self.assertEqual(response.status_int, 200)
        self.assertIn('image/x-icon', response.headers['Content-Type'])
        self.assertIsNotNone(response.body)


class TestConceptSchemeView(unittest.TestCase):
    def setUp(self):
        self.config = testing.setUp()
        self.regis = Registry()
        self.regis.register_provider(trees)
        self.request = testing.DummyRequest()
        self.request.accept = 'text/html'
        self.request.data_managers = {
            'skos_manager': DummySKOSManager(),
            'conceptscheme_manager': DummyConceptschemeManager(),
            'audit_manager': DummyAuditManager(),
        }
        self.request.skos_registry = self.regis

    def tearDown(self):
        testing.tearDown()

    def test_conceptschemes_view(self):
        atramhasisview = AtramhasisView(self.request)
        res = atramhasisview.conceptschemes_view()
        self.assertIn('conceptschemes', res)
        self.assertEqual(len(res['conceptschemes']), 1)
        cs = res['conceptschemes'][0]
        self.assertIn('id', cs)
        self.assertIn('conceptscheme', cs)

    def test_conceptscheme_view(self):
        self.request.matchdict['scheme_id'] = 'TREES'
        atramhasisview = AtramhasisView(self.request)
        res = atramhasisview.conceptscheme_view()
        self.assertIsNotNone(res)
        self.assertIsNotNone(res['conceptscheme'])
        self.assertEqual(res['conceptscheme']['title'], 'TREES')
        self.assertEqual(res['conceptscheme']['scheme_id'], 'TREES')
        self.assertEqual(res['conceptscheme']['uri'], 'urn:x-skosprovider:trees')
        self.assertIsNotNone(res['conceptscheme']['labels'])
        self.assertIsNotNone(res['conceptscheme']['notes'])
        self.assertIsNotNone(res['conceptscheme']['top_concepts'])

    def test_conceptscheme_view_language(self):
        self.request.matchdict['scheme_id'] = 'TREES'
        self.request.skos_registry.providers['TREES'].metadata[
            'atramhasis.force_display_label_language'
        ] = 'nl'

        atramhasisview = AtramhasisView(self.request)
        res = atramhasisview.conceptscheme_view()
        self.assertIsNotNone(res)
        self.assertEqual(res['locale'], 'nl')


class TestConceptView(unittest.TestCase):
    def setUp(self):
        self.config = testing.setUp()
        self.config.add_route(
            'skosprovider.c',
            pattern='/conceptschemes/{scheme_id}/c/{c_id}',
            accept='text/html',
            request_method='GET',
        )
        self.request = testing.DummyRequest()
        self.request.accept = 'text/html'
        self.regis = Registry()
        self.regis.register_provider(provider(1))
        self.request.data_managers = {
            'skos_manager': DummySKOSManager(),
            'conceptscheme_manager': DummyConceptschemeManager(),
            'audit_manager': DummyAuditManager(),
        }

    def tearDown(self):
        testing.tearDown()

    def test_passing_view(self):
        request = self.request
        request.matchdict['scheme_id'] = 'TREES'
        request.matchdict['c_id'] = '1'
        request.skos_registry = self.regis
        atramhasisview = AtramhasisView(request)
        info = atramhasisview.concept_view()
        self.assertIsNotNone(info['concept'])
        self.assertEqual(info['conceptType'], 'Concept')
        self.assertEqual(info['scheme_id'], 'TREES')

    def test_passing_view_with_languague(self):
        request = self.request
        request.matchdict['scheme_id'] = 'TREES'
        request.matchdict['c_id'] = '1'
        request.skos_registry = self.regis
        request.skos_registry.providers['TREES'].metadata = {
            'atramhasis.force_display_label_language': 'nl'
        }
        atramhasisview = AtramhasisView(request)
        info = atramhasisview.concept_view()
        self.assertIsNotNone(info['concept'])
        self.assertEqual(info['locale'], 'nl')

    def test_passing_collection_view(self):
        request = self.request
        request.matchdict['scheme_id'] = 'TREES'
        request.matchdict['c_id'] = '3'
        request.skos_registry = self.regis
        atramhasisview = AtramhasisView(request)
        info = atramhasisview.concept_view()
        self.assertIsNotNone(info['concept'])
        self.assertEqual(info['conceptType'], 'Collection')
        self.assertEqual(info['scheme_id'], 'TREES')

    def test_provider_not_found(self):
        request = self.request
        request.matchdict['scheme_id'] = 'ZZ'
        request.matchdict['c_id'] = '1'
        request.skos_registry = self.regis
        atramhasisview = AtramhasisView(request)
        with self.assertRaises(ConceptSchemeNotFoundException):
            atramhasisview.concept_view()

    def test_not_found(self):
        request = self.request
        request.matchdict['scheme_id'] = 'TREES'
        request.matchdict['c_id'] = '666'
        request.skos_registry = self.regis
        atramhasisview = AtramhasisView(request)
        with self.assertRaises(ConceptNotFoundException):
            atramhasisview.concept_view()

    def test_no_type(self):
        request = self.request
        request.matchdict['scheme_id'] = 'TREES'
        request.matchdict['c_id'] = '555'
        request.skos_registry = self.regis
        atramhasisview = AtramhasisView(request)
        info = atramhasisview.concept_view()
        self.assertEqual(info.status_int, 500)


class TestSearchResultView(unittest.TestCase):
    def setUp(self):
        self.config = testing.setUp()
        self.regis = Registry()
        self.regis.register_provider(trees)
        self.request = testing.DummyRequest()
        self.request.data_managers = {
            'skos_manager': None,
            'conceptscheme_manager': None,
            'audit_manager': None,
        }

    def tearDown(self):
        testing.tearDown()

    def test_find_by_label(self):
        self.request.matchdict['scheme_id'] = 'TREES'
        self.request.params = MultiDict()
        self.request.params.add('label', 'De Paardekastanje')
        self.request.params.add('_LOCALE_', 'nl')
        self.request.skos_registry = self.regis
        atramhasisview = AtramhasisView(self.request)
        info = atramhasisview.search_result()
        self.assertIsNotNone(info['concepts'])
        concept = info['concepts'][0]
        self.assertIsNotNone(concept)
        self.assertEqual(concept['label'], 'De Paardekastanje')
        self.assertEqual(info['scheme_id'], 'TREES')

    def test_find_by_concept(self):
        self.request.matchdict['scheme_id'] = 'TREES'
        self.request.params = MultiDict()
        self.request.params.add('type', 'concept')
        self.request.params.add('_LOCALE_', 'nl')
        self.request.skos_registry = self.regis
        atramhasisview = AtramhasisView(self.request)
        info = atramhasisview.search_result()
        self.assertIsNotNone(info['concepts'])
        concept = info['concepts'][0]
        self.assertIsNotNone(concept)
        self.assertEqual(info['scheme_id'], 'TREES')

    def test_no_querystring(self):
        self.request.matchdict['scheme_id'] = 'TREES'
        self.request.params = MultiDict()
        self.request.skos_registry = self.regis
        atramhasisview = AtramhasisView(self.request)
        info = atramhasisview.search_result()
        self.assertIsNotNone(info['concepts'])
        self.assertEqual(len(info['concepts']), 3)

    def test_no_schema(self):
        self.request.matchdict['scheme_id'] = 'GG'
        self.request.params = MultiDict()
        self.request.skos_registry = self.regis
        atramhasisview = AtramhasisView(self.request)
        info = atramhasisview.search_result()
        self.assertEqual(info.status_int, 404)


class TestCsvView(unittest.TestCase):
    def setUp(self):
        self.config = testing.setUp()
        self.request = testing.DummyRequest()
        self.request.accept = '*/*'
        self.regis = Registry()
        self.regis.register_provider(provider(1))
        self.request.skos_registry = self.regis
        self.request.data_managers = {
            'skos_manager': DummySKOSManager(),
            'conceptscheme_manager': DummyConceptschemeManager(),
            'audit_manager': DummyAuditManager(),
        }

    def tearDown(self):
        testing.tearDown()

    def test_csv(self):
        self.request.matchdict['scheme_id'] = 'TREES'
        self.request.params = MultiDict()
        atramhasisview = AtramhasisView(self.request)
        res = atramhasisview.results_csv()
        self.assertEqual(res['filename'], 'atramhasis_export')
        self.assertIsInstance(res['header'], list)
        self.assertIsInstance(res['rows'], list)
        self.assertEqual(2, len(res['rows']))

    def test_csv_label(self):
        self.request.matchdict['scheme_id'] = 'TREES'
        self.request.params = MultiDict()
        self.request.params.add('label', 'De Paardekastanje')
        atramhasisview = AtramhasisView(self.request)
        res = atramhasisview.results_csv()
        self.assertEqual(res['filename'], 'atramhasis_export')
        self.assertIsInstance(res['header'], list)
        self.assertIsInstance(res['rows'], list)
        self.assertEqual(2, len(res['rows']))

    def test_csv_type(self):
        self.request.matchdict['scheme_id'] = 'TREES'
        self.request.params = MultiDict()
        self.request.params.add('type', 'concept')
        atramhasisview = AtramhasisView(self.request)
        res = atramhasisview.results_csv()
        self.assertEqual(res['filename'], 'atramhasis_export')
        self.assertIsInstance(res['header'], list)
        self.assertIsInstance(res['rows'], list)
        self.assertEqual(2, len(res['rows']))


class TestLocaleView(unittest.TestCase):
    def setUp(self):
        self.config = testing.setUp()
        self.regis = Registry()
        self.regis.register_provider(trees)
        config = testing.setUp()
        config.add_route('home', 'foo')
        config.add_settings(settings)
        self.request = testing.DummyRequest()
        self.request.data_managers = {
            'skos_manager': None,
            'conceptscheme_manager': None,
            'audit_manager': None,
        }

    def tearDown(self):
        testing.tearDown()

    def test_default_locale(self):
        config_default_lang = settings.get('pyramid.default_locale_name')
        self.request.referer = None
        self.request.skos_registry = self.regis
        atramhasisview = AtramhasisView(self.request)
        res = atramhasisview.set_locale_cookie()
        self.assertTrue(
            (res.headers.get('Set-Cookie')).startswith(
                '_LOCALE_=' + config_default_lang
            )
        )

    def test_unsupported_lang(self):
        config_default_lang = settings.get('pyramid.default_locale_name')
        self.request.GET['language'] = 'XX'
        self.request.referer = None
        self.request.skos_registry = self.regis
        atramhasisview = AtramhasisView(self.request)
        res = atramhasisview.set_locale_cookie()
        self.assertTrue(
            (res.headers.get('Set-Cookie')).startswith(
                '_LOCALE_=' + config_default_lang
            )
        )

    def test_locale(self):
        testlang = 'it'
        self.request.GET['language'] = testlang
        self.request.referer = None
        self.request.skos_registry = self.regis
        atramhasisview = AtramhasisView(self.request)
        res = atramhasisview.set_locale_cookie()
        self.assertTrue(
            (res.headers.get('Set-Cookie')).startswith('_LOCALE_=' + testlang)
        )

    def test_locale_uppercase(self):
        testlang = 'it'
        self.request.GET['language'] = testlang.upper()
        self.request.referer = None
        self.request.skos_registry = self.regis
        atramhasisview = AtramhasisView(self.request)
        res = atramhasisview.set_locale_cookie()
        self.assertTrue(
            (res.headers.get('Set-Cookie')).startswith('_LOCALE_=' + testlang)
        )

    def test_referer(self):
        testlang = 'it'
        testurl = 'https://www.foo.bar'
        self.request.GET['language'] = testlang.upper()
        self.request.referer = testurl
        self.request.skos_registry = self.regis
        atramhasisview = AtramhasisView(self.request)
        res = atramhasisview.set_locale_cookie()
        self.assertEqual(res.status, '302 Found')
        self.assertEqual(res.location, testurl)


class TestHtmlTreeView(unittest.TestCase):
    def setUp(self):
        self.config = testing.setUp()
        self.regis = Registry()
        self.regis.register_provider(trees)
        self.request = testing.DummyRequest()
        self.request.data_managers = {
            'skos_manager': None,
            'conceptscheme_manager': None,
            'audit_manager': None,
        }

    def tearDown(self):
        testing.tearDown()


class TestAdminView(unittest.TestCase):
    def setUp(self):
        self.config = testing.setUp()
        self.regis = Registry()
        self.regis.register_provider(trees)

    def tearDown(self):
        testing.tearDown()

    def test_no_registry(self):
        request = testing.DummyRequest()
        with self.assertRaises(SkosRegistryNotFoundException):
            AtramhasisAdminView(request)

    def test_passing_view(self):
        request = testing.DummyRequest()
        request.skos_registry = self.regis
        atramhasis_admin_view = AtramhasisAdminView(request)
        info = atramhasis_admin_view.admin_view()
        self.assertIsNotNone(info)
        self.assertTrue('admin' in info)

    def test_invalidate_scheme_tree(self):
        request = testing.DummyRequest()
        request.matchdict['scheme_id'] = 'TREES'
        request.skos_registry = self.regis
        atramhasis_admin_view = AtramhasisAdminView(request)
        info = atramhasis_admin_view.invalidate_scheme_tree()
        self.assertIsNotNone(info)


class TestViewFunctions(unittest.TestCase):
    def test_labels_to_string(self):
        labels = [
            Label(label='De Paardekastanje', language_id='nl'),
            Label(label='la châtaigne', language_id='fr'),
        ]
        s = labels_to_string(labels, 'prefLabel')
        self.assertEqual('De Paardekastanje (nl), la châtaigne (fr)', s)

    def test_get_definition(self):
        notes = [
            Note(note='test', language_id='nl', notetype_id='note'),
            Note(note='test2', language_id='nl', notetype_id='definition'),
        ]
        s = get_definition(notes)
        self.assertEqual('test2', s)

    def test_get_public_conceptschemes(self):
        regis = Registry()
        regis.register_provider(trees)
        regis.register_provider(hidden_provider(2))
        regis.register_provider(external_provider(3))
        conceptschemes = get_public_conceptschemes(regis)
        self.assertEqual(1, len(conceptschemes))


class TestListViews(unittest.TestCase):
    def setUp(self):
        self.config = testing.setUp()
        self.request = testing.DummyRequest()
        self.request.data_managers = {
            'skos_manager': DummySKOSManager(),
        }

    def test_get_list(self):
        request = self.request
        atramhasis_list_view = AtramhasisListView(request)
        labellist = atramhasis_list_view.get_list(LabelType)
        self.assertIsNotNone(labellist)
        self.assertIsNotNone(labellist[0])

    def test_labeltype_list_view(self):
        request = self.request
        atramhasis_list_view = AtramhasisListView(request)
        labellist = atramhasis_list_view.labeltype_list_view()
        self.assertIsNotNone(labellist)
        self.assertIn({'key': 'prefLabel', 'label': 'prefLabel'}, labellist)


class TestAtramhasisCrudView(unittest.TestCase):
    class SKOSRegistry:
        pass

    class SKOSManager:
        def get_all_label_types(self):
            return [
                LabelType(name='prefLabel', description=''),
                LabelType(name='sortLabel', description=''),
            ]

        def get_next_cid(self, _, __):
            return 'next_cid'

        def save(self, concept):
            if concept.id is None:
                concept.id = 1
            concept.conceptscheme = ConceptScheme(uri='urn:x-skosprovider:test')
            return concept

        def get_thing(self, concept_id, scheme_id):
            raise NoResultFound()

    class LanguagesManager:
        def count_languages(self, _):
            return 1

    class Provider(SQLAlchemyProvider):
        metadata = {'subject': 'stub'}
        conceptscheme_id = 'cs_id'
        uri_generator = DefaultUrnGenerator('voc-id')

        def __init__(self):
            pass

    def setUp(self):
        pyramid_settings = Settings(settings)
        config = testing.setUp(settings=pyramid_settings)
        atramhasis.load_app(config)
        self.request = DummyRequest()
        self.request.application_url = 'http://localhost:6543'
        apply_request_extensions(self.request)
        self.request.data_managers = {
            'skos_manager': TestAtramhasisCrudView.SKOSManager(),
            'conceptscheme_manager': None,
            'audit_manager': None,
            'languages_manager': TestAtramhasisCrudView.LanguagesManager(),
        }
        self.view = AtramhasisCrud(self.request)
        self.view.skos_registry = TestAtramhasisCrudView.SKOSRegistry()
        self.view.provider = TestAtramhasisCrudView.Provider()
        self.view.scheme_id = 's_id'

    def get_concept_json(self):
        return {
            'type': 'concept',
            'broader': [],
            'narrower': [],
            'related': [],
            'labels': [
                {'type': 'prefLabel', 'language': 'en', 'label': 'The Larch'},
                {'type': 'sortLabel', 'language': 'en', 'label': 'a'},
            ],
            'notes': [],
            'sources': [{'citation': 'Citation'}],
        }

    def test_add_concept(self):
        self.request.json_body = self.get_concept_json()
        concept = self.view.add_concept()
        self.assertIsInstance(concept, SkosConcept)
        self.assertEqual('next_cid', concept.id)
        self.assertEqual('urn:x-skosprovider:voc-id:next_cid', concept.uri)

    def test_add_concept_manual_id_strategy(self):
        strategy = IDGenerationStrategy.MANUAL
        self.view.provider.metadata['atramhasis.id_generation_strategy'] = strategy
        self.request.json_body = self.get_concept_json()
        self.request.json_body['id'] = 'manual'

        concept = self.view.add_concept()
        self.assertIsInstance(concept, SkosConcept)
        self.assertEqual('manual', concept.id)

        del self.request.json_body['id']
        with pytest.raises(ValidationError):
            self.view.add_concept()

        # add_concept should fail without id, edit_concept should not because
        # no id validation happens
        self.request.matchdict['c_id'] = 'manual'
        db_concept = Concept()
        db_concept.conceptscheme = ConceptScheme(id=1, uri='urn:x-skosprovider:trees')
        self.request.data_managers['skos_manager'].get_thing = lambda *_: db_concept
        self.view.edit_concept()

    def test_add_provider(self):
        self.request.openapi_validated = Mock()
        self.request.skos_registry = Registry()
        view = 'atramhasis.views.crud'

        with (
            mock.patch(f'{view}.provider.create_provider', autospec=True) as processor,
            mock.patch(
                f'{view}.utils.db_provider_to_skosprovider', autospec=True
            ) as renderer,
        ):
            response = self.view.add_provider()
            self.assertEqual(201, self.request.response.status_code)
            processor.assert_called()
            renderer.assert_called()
            self.assertEqual(response, renderer.return_value)

    def test_update_provider(self):
        self.request.openapi_validated = Mock()
        self.request.matchdict = {'id': 1}
        view = 'atramhasis.views.crud'

        with (
            mock.patch(f'{view}.provider.update_provider', autospec=True) as processor,
            mock.patch(
                f'{view}.utils.db_provider_to_skosprovider', autospec=True
            ) as renderer,
        ):
            response = self.view.update_provider()
            processor.assert_called()
            renderer.assert_called()
            self.assertEqual(response, renderer.return_value)

    def test_delete_provider(self):
        self.request.matchdict = {'id': 1}
        view = 'atramhasis.views.crud'

        with mock.patch(f'{view}.provider.delete_provider', autospec=True) as processor:
            response = self.view.delete_provider()
            processor.assert_called()
            self.assertIsInstance(response, HTTPNoContent)



"""Add conceptscheme counts

Revision ID: b04fd493106b
Revises: 3bcf11802900
Create Date: 2016-11-02 07:36:17.940810

"""
import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "b04fd493106b"
down_revision = "3bcf11802900"


def upgrade():
    op.create_table(
        "conceptscheme_counts",
        sa.Column("id", sa.Integer(), primary_key=True, autoincrement=True),
        sa.Column("conceptscheme_id", sa.String(), nullable=False),
        sa.Column("counted_at", sa.DateTime(), nullable=False),
        sa.Column("triples", sa.Integer, nullable=False),
        sa.Column("conceptscheme_triples", sa.Integer, nullable=False),
        sa.Column("avg_concept_triples", sa.Integer, nullable=False),
    )


def downgrade():
    op.drop_table("conceptscheme_counts")



import argparse
import logging
from builtins import input

from pyramid.paster import get_appsettings
from pyramid.paster import setup_logging
from sqlalchemy.sql.expression import text

from atramhasis import utils

log = logging.getLogger(__name__)


def delete_scheme(session, scheme_id):
    concept_ids = session.execute(
        text(f'select id from concept where conceptscheme_id={scheme_id}')
    )
    for row in concept_ids:
        concept_id = row[0]
        delete_concept(concept_id, session)

    session.execute(
        text(
            f'delete from note where note.id in'
            f'(select note_id from conceptscheme_note '
            f'where conceptscheme_id={scheme_id})'
        )
    )
    session.execute(
        text(
            f'delete from source where source.id in'
            f'(select source_id from conceptscheme_source '
            f'where conceptscheme_id={scheme_id})'
        )
    )
    session.execute(
        text(
            f'delete from label where label.id in'
            f'(select label_id from conceptscheme_label '
            f'where conceptscheme_id={scheme_id})'
        )
    )
    session.execute(text(f'delete from conceptscheme where id = {scheme_id}'))


def delete_concept(concept_id, session):
    session.execute(
        text(
            f'delete from note where note.id in'
            f'(select note_id from concept_note where concept_id={concept_id})'
        )
    )
    session.execute(
        text(
            f'delete from source where source.id in'
            f'(select source_id from concept_source '
            f'where concept_id={concept_id})'
        )
    )
    session.execute(
        text(
            f'delete from label where label.id in'
            f'(select label_id from concept_label '
            f'where concept_id={concept_id})'
        )
    )
    delete_child_concepts(concept_id, session)
    session.execute(text(f'delete from concept where id = {concept_id}'))


def delete_child_concepts(concept_id, session):
    select_children = (
        text(
            f'select collection_id_narrower as child '
            f'from concept_hierarchy_collection '
            f'where concept_id_broader = {concept_id} '
            f'union '
            f'select concept_id_narrower as child '
            f'from concept_hierarchy_concept '
            f'where concept_id_broader = {concept_id}'
        )
    )
    children = session.execute(select_children)
    for row in children:
        child_id = row[0]
        delete_concept(child_id, session)


def main():
    parser = argparse.ArgumentParser(
        description="Delete a conceptscheme. ",
        usage="remove_schema development.ini --id=1")
    parser.add_argument('settings_file',
                        help="<The location of the settings file>#<app-name>")
    parser.add_argument("--id", type=int, required=True,
                        help="the conceptscheme id")
    parser.add_argument("--no-input", action='store_true',
                        help="Don't stop script for user input")
    args = parser.parse_args()

    config_uri = args.settings_file
    setup_logging(config_uri)
    settings = get_appsettings(config_uri)
    print(
        f"The conceptscheme with id {args.id} will be deleted"
    )
    if not args.no_input:
        input("Press [Enter] to continue.")

    with utils.db_session(settings) as session:
        delete_scheme(session, args.id)


if __name__ == '__main__':  # pragma: no cover
    main()



import re

from atramhasis.errors import ConceptSchemeNotFoundException

"""
Thid module is used when blocking operations on a certain Concept or Collection
that might be used in external applications.

:versionadded: 0.4.0
"""


class ProtectedResourceEvent:
    """
    Event triggered when calling a protected operation on a resource
    """

    def __init__(self, uri, request):
        self.uri = uri
        self.request = request


def protected_operation(fn):
    """
    use this decorator to prevent an operation from being executed
    when the related resource is still in use
    """

    def advice(parent_object, *args, **kw):
        request = parent_object.request
        url = request.path
        match = re.compile(r'/conceptschemes/(\w+)/c/(\w+)').match(url)
        scheme_id = match.group(1)
        c_id = match.group(2)
        provider = request.skos_registry.get_provider(scheme_id)
        if not provider:
            raise ConceptSchemeNotFoundException(scheme_id)
        uri = provider.uri_generator.generate(id=c_id)
        event = ProtectedResourceEvent(uri, request)
        parent_object.request.registry.notify(event)
        return fn(parent_object, *args, **kw)

    return advice


class ProtectedResourceException(Exception):
    """
    raise this exception when the resource is still used somewhere

    referenced_in should contain locations where the resource is still referenced
    """

    def __init__(self, value, referenced_in):
        self.value = value
        self.referenced_in = referenced_in

    def __str__(self):
        return repr(self.value)



import unittest
from unittest.mock import MagicMock
from unittest.mock import Mock

from atramhasis.errors import ConceptSchemeNotFoundException
from atramhasis.protected_resources import ProtectedResourceEvent
from atramhasis.protected_resources import ProtectedResourceException
from atramhasis.protected_resources import protected_operation


class DummyParent:

    def __init__(self):
        self.request = MagicMock()
        self.request.path = '/conceptschemes/GEOGRAPHY/c/9'

    @protected_operation
    def protected_dummy(self):
        return 'dummy ok'


class ProtectedTests(unittest.TestCase):

    def setUp(self):
        pass

    def test_protected_resource_event(self):
        event = ProtectedResourceEvent('urn:test', 'request')
        self.assertEqual('urn:test', event.uri)
        self.assertEqual('request', event.request)

    def test_protected_resource_exception(self):
        referenced_in = ['urn:someobject', 'https://test.test.org/object/2']
        error = ProtectedResourceException('test_msg', referenced_in)
        self.assertIsNotNone(error)
        self.assertEqual("'test_msg'", str(error))

    def test_protected_event(self):
        dummy = DummyParent()
        provider = MagicMock()
        provider.uri_generator.generate = lambda id: 'urn:x-vioe:geography:9'
        dummy.request.skos_registry.get_provider = lambda scheme_id: provider
        notify_mock = Mock()
        dummy.request.registry.notify = notify_mock
        dummy.protected_dummy()
        notify_call = notify_mock.mock_calls[0]
        self.assertEqual('urn:x-vioe:geography:9', notify_call[1][0].uri)

    def test_protected_event_error(self):
        dummy = DummyParent()
        dummy.request.skos_registry.get_provider = lambda scheme_id: None
        notify_mock = Mock()
        dummy.request.registry.notify = notify_mock
        self.assertRaises(ConceptSchemeNotFoundException, dummy.protected_dummy)



import contextlib
import os
import unittest

from alembic import command
from alembic.config import Config
from pyramid.paster import get_appsettings
from skosprovider.providers import DictionaryProvider
from skosprovider.registry import Registry
from skosprovider.skos import ConceptScheme
from skosprovider.uri import UriPatternGenerator
from skosprovider_sqlalchemy.models import Concept
from skosprovider_sqlalchemy.providers import SQLAlchemyProvider
from skosprovider_sqlalchemy.utils import import_provider
from sqlalchemy import engine_from_config
from sqlalchemy import text
from sqlalchemy.exc import OperationalError
from sqlalchemy.exc import ProgrammingError
from sqlalchemy.orm import sessionmaker

from atramhasis import skos
from atramhasis.cache import list_region
from atramhasis.cache import tree_region
from atramhasis.data.models import IDGenerationStrategy
from fixtures import data
from fixtures import materials as material_data

TEST_DIR = os.path.dirname(__file__)
SETTINGS = get_appsettings(os.path.join(TEST_DIR, '..', 'tests', 'conf_test.ini'))
db_setup = False
db_filled = False

# No test should want caching
tree_region.configure('dogpile.cache.null', replace_existing_backend=True)
list_region.configure('dogpile.cache.null', replace_existing_backend=True)


def get_alembic_config():
    alembic_config = Config(os.path.join(TEST_DIR, '..', 'alembic.ini'))
    alembic_config.set_main_option(
        "script_location",
        os.path.join(TEST_DIR, '..', 'atramhasis', 'alembic')
    )
    alembic_config.set_main_option(
        "ini_location",
        os.path.join(TEST_DIR, '..', 'tests', 'conf_test.ini')
    )
    return alembic_config


ALEMBIC_CONFIG = get_alembic_config()


def setup_db(guarantee_empty=False):
    global db_setup, db_filled
    if not db_setup or (guarantee_empty and db_filled):
        _reset_db()
        command.upgrade(ALEMBIC_CONFIG, 'head')
        db_setup = True
        db_filled = False


def _reset_db():
    engine = engine_from_config(SETTINGS, prefix='sqlalchemy.')
    session = sessionmaker(bind=engine)()
    try:
        session.execute(text("DELETE FROM concept_note"))
        session.execute(text("DELETE FROM note"))
        session.execute(text("DELETE FROM concept_label"))
        session.execute(text("DELETE FROM label"))
    except (ProgrammingError, OperationalError):
        """The tables may not exist if it's first time."""
    session.commit()
    session.close()
    command.downgrade(ALEMBIC_CONFIG, 'base')
    engine.dispose()


def fill_db():
    global db_filled
    if not db_filled:
        from fixtures.data import trees
        from skosprovider_sqlalchemy.models import ConceptScheme
        with db_session() as session:
            import_provider(trees,
                            session,
                            ConceptScheme(id=1, uri='urn:x-skosprovider:trees'))
            import_provider(material_data.materials,
                            session,
                            ConceptScheme(id=4, uri='urn:x-vioe:materials'))
            import_provider(data.geo,
                            session,
                            ConceptScheme(id=2, uri='urn:x-vioe:geography'))
            import_provider(
                DictionaryProvider(
                    {'id': 'MISSING_LABEL', 'default_language': 'nl'},
                    [{'id': '1', 'uri': 'urn:x-skosprovider:test/1'},
                     {
                         'id': '2',
                         'uri': 'urn:x-skosprovider:test/2',
                         'labels': [
                             {'type': 'prefLabel', 'language': 'nl', 'label': 'label'}
                         ],
                     }]
                ),
                session,
                ConceptScheme(id=9, uri='urn:x-vioe:test')
            )
            import_provider(
                DictionaryProvider(
                    {'id': 'manual-ids', 'default_language': 'nl'},
                    [{'id': 'manual-1', 'uri': 'urn:x-skosprovider:manual/manual-1'},
                     {
                         'id': 'manual-2',
                         'uri': 'urn:x-skosprovider:manual/manual-2',
                         'labels': [
                             {'type': 'prefLabel', 'language': 'nl', 'label': 'label'}
                         ],
                     },
                     {
                         'id': 'https://id.manual.org/manual/68',
                         'uri': 'https://id.manual.org/manual/68',
                         'labels': [
                             {'type': 'prefLabel', 'language': 'nl', 'label': 'handmatig'}
                         ],
                     }]
                ),
                session,
                ConceptScheme(id=10, uri='urn:x-vioe:manual'),
            )
            session.add(ConceptScheme(id=3, uri='urn:x-vioe:styles'))
            for scheme_id in (5, 6, 7, 8):
                session.add(
                    ConceptScheme(id=scheme_id, uri=f'urn:dummy-{scheme_id}')
                )
        db_filled = True


class DbTest(unittest.TestCase):
    engine = None
    connection = None

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.engine = engine_from_config(SETTINGS, prefix='sqlalchemy.')
        cls.connection = cls.engine.connect()

    @classmethod
    def tearDownClass(cls):
        cls.connection.close()
        cls.engine.dispose()
        super().tearDownClass()

    def setUp(self):
        super().setUp()
        self.transaction = self.connection.begin()
        self.session = sessionmaker(bind=self.connection)()

    def tearDown(self):
        self.session.close()
        self.transaction.rollback()

    def insert(self, db_object):
        self.session.add(db_object)
        self.session.flush()
        self.session.refresh(db_object)

    def update(self, db_object):
        self.session.merge(db_object)
        self.session.flush()
        self.session.refresh(db_object)


@contextlib.contextmanager
def db_session():
    engine = engine_from_config(SETTINGS, prefix='sqlalchemy.')
    session_maker = sessionmaker(bind=engine)
    session = session_maker()
    try:
        yield session
        session.commit()
    except Exception:
        session.rollback()
        raise
    finally:
        session.close()


def create_registry(request):
    registry = Registry(instance_scope='threaded_thread')
    trees = SQLAlchemyProvider(
        {'id': 'TREES', 'conceptscheme_id': 1},
        request.db
    )

    geo = SQLAlchemyProvider(
        {'id': 'GEOGRAPHY', 'conceptscheme_id': 2},
        request.db,
        uri_generator=UriPatternGenerator('urn:x-vioe:geography:%s')
    )

    styles = SQLAlchemyProvider(
        {'id': 'STYLES', 'conceptscheme_id': 3},
        request.db
    )

    materials = SQLAlchemyProvider(
        {'id': 'MATERIALS', 'conceptscheme_id': 4},
        request.db,
        uri_generator=UriPatternGenerator('urn:x-vioe:materials:%s')
    )
    test = DictionaryProvider(
        {
            'id': 'TEST',
            'default_language': 'nl',
            'subject': ['biology']
        },
        [data.larch, data.chestnut, data.species],
        concept_scheme=ConceptScheme('http://id.trees.org')
    )
    missing_label = SQLAlchemyProvider(
        {'id': 'MISSING_LABEL', 'conceptscheme_id': 9},
        request.db
    )
    manual_ids = SQLAlchemyProvider(
        {
            'id': 'manual-ids',
            'conceptscheme_id': 10,
            'atramhasis.id_generation_strategy': IDGenerationStrategy.MANUAL
        },
        request.db
    )

    registry.register_provider(trees)
    registry.register_provider(geo)
    registry.register_provider(styles)
    registry.register_provider(materials)
    registry.register_provider(test)
    registry.register_provider(missing_label)
    registry.register_provider(manual_ids)

    skos.register_providers_from_db(registry, request.db)

    return registry



import logging

import requests
from atramhasis import skos
from cachecontrol import CacheControl
from cachecontrol.heuristics import ExpiresAfter
from skosprovider.registry import Registry
from skosprovider_getty.providers import AATProvider
from skosprovider_getty.providers import TGNProvider

log = logging.getLogger(__name__)
LICENSES = [
    'https://creativecommons.org/licenses/by/4.0/',
    'https://data.vlaanderen.be/doc/licentie/modellicentie-gratis-hergebruik/v1.0'
]


def create_registry(request):
    try:
        registry = Registry(instance_scope='threaded_thread')

        getty_session = CacheControl(requests.Session(), heuristic=ExpiresAfter(weeks=1))

        aat = AATProvider(
            {'id': 'AAT', 'subject': ['external']},
            session=getty_session
        )

        tgn = TGNProvider(
            {'id': 'TGN', 'subject': ['external']},
            session=getty_session
        )

        registry.register_provider(aat)
        registry.register_provider(tgn)
        skos.register_providers_from_db(registry, request.db)

        return registry
    except AttributeError:
        log.exception("Attribute error during creation of Registry.")
        raise



"""
This module adds DataManagers for Atramhasis. These are service layer objects
that abstract all interactions with the database away from the views.

:versionadded: 0.4.1
"""

import uuid
from datetime import date
from datetime import datetime
from typing import List

import dateutil.relativedelta
import sqlalchemy as sa
from skosprovider_sqlalchemy.models import Collection
from skosprovider_sqlalchemy.models import Concept
from skosprovider_sqlalchemy.models import ConceptScheme
from skosprovider_sqlalchemy.models import Label
from skosprovider_sqlalchemy.models import LabelType
from skosprovider_sqlalchemy.models import Language
from skosprovider_sqlalchemy.models import Match
from skosprovider_sqlalchemy.models import MatchType
from skosprovider_sqlalchemy.models import Thing
from skosprovider_sqlalchemy.models import collection_concept
from skosprovider_sqlalchemy.models import concept_hierarchy_concept
from sqlalchemy import desc
from sqlalchemy import func
from sqlalchemy import select
from sqlalchemy.orm import Session
from sqlalchemy.orm import joinedload

from atramhasis.data import popular_concepts
from atramhasis.data.models import ConceptVisitLog
from atramhasis.data.models import ConceptschemeCounts
from atramhasis.data.models import IDGenerationStrategy
from atramhasis.data.models import Provider
from atramhasis.scripts import delete_scheme


class DataManager:
    """
    A DataManager abstracts all interactions with the database for a certain model.
    """

    def __init__(self, session: Session) -> None:
        self.session: Session = session


class ConceptSchemeManager(DataManager):
    """
    A :class:`DataManager` for
    :class:`ConceptSchemes <skosprovider_sqlalchemy.models.ConceptScheme>.`
    """

    def __init__(self, session):
        super().__init__(session)

    def get(self, conceptscheme_id):
        """

        :param conceptscheme_id: a concepscheme id
        :return: the concepscheme for the given id
        """
        return self.session.execute(
            select(ConceptScheme).filter(ConceptScheme.id == conceptscheme_id)
        ).scalar_one()

    def find(self, conceptscheme_id, query):
        """
        Find concepts and collections in this concept scheme.

        :param conceptscheme_id: a concepscheme id
        :param query: A python dictionary containing query parameters.
        :returns: A :class:`list` of
            :class:`skosprovider_sqlalchemy.models.Thing` instances.
        """
        db_query = (
            select(Thing)
            .options(joinedload(Thing.labels))
            .filter(Thing.conceptscheme_id == conceptscheme_id)
        )
        if 'type' in query and query['type'] in ['concept', 'collection']:
            db_query = db_query.filter(Thing.type == query['type'])
        if 'label' in query:
            db_query = db_query.filter(
                Thing.labels.any(Label.label.ilike('%' + query['label'].lower() + '%'))
            )
        return self.session.execute(db_query).unique().scalars().all()

    def get_concepts_for_scheme_tree(self, conceptscheme_id):
        """

        :param conceptscheme_id:  a concepscheme id
        :return: all concepts for the scheme_tree
        """
        return (
            self.session.execute(
                select(Concept).filter(
                    Concept.conceptscheme_id == conceptscheme_id,
                    ~Concept.broader_concepts.any(),
                    ~Collection.member_of.any(),
                )
            )
            .scalars()
            .all()
        )

    def get_collections_for_scheme_tree(self, conceptscheme_id):
        """

        :param conceptscheme_id: a concepscheme id
        :return: all collections for the scheme_tree
        """
        return (
            self.session.execute(
                select(Collection).filter(
                    Collection.conceptscheme_id == conceptscheme_id,
                    ~Collection.broader_concepts.any(),
                    ~Collection.member_of.any(),
                )
            )
            .scalars()
            .all()
        )

    def get_all(self, conceptscheme_id):
        """
        Get all concepts and collections in this concept scheme.

        :param conceptscheme_id: a concepscheme id
        :returns: A :class:`list` of
            :class:`skosprovider_sqlalchemy.models.Thing` instances.
        """
        all_results = (
            self.session.execute(
                select(Thing)
                .options(joinedload(Thing.labels))
                .filter(Thing.conceptscheme_id == conceptscheme_id)
            )
            .unique()
            .scalars()
            .all()
        )
        return all_results

    def save(self, conceptscheme):
        """

        :param conceptscheme: conceptscheme to save
        :return: saved conceptscheme
        """
        self.session.merge(conceptscheme)
        self.session.flush()
        return conceptscheme


class SkosManager(DataManager):
    """
    A :class:`DataManager` for
    :class:`Concepts and Collections <skosprovider_sqlalchemy.models.Thing>.`
    """

    def __init__(self, session):
        super().__init__(session)

    def get_thing(self, concept_id, conceptscheme_id):
        """

        :param concept_id: a concept id
        :param conceptscheme_id: a conceptscheme id
        :return: the selected thing (Concept or Collection)
        """
        return self.session.execute(
            select(Thing).filter(
                Thing.concept_id == concept_id,
                Thing.conceptscheme_id == conceptscheme_id,
            )
        ).scalar_one()

    def save(self, thing):
        """

        :param thing: thing to save
        :return: saved thing
        """
        self.session.add(thing)
        self.session.flush()
        # We refresh to guarantee the data types are correct
        # For example, concept_id might be an integer when generation is set to
        # numeric, but this should be a string.
        self.session.refresh(thing)
        return thing

    def change_type(self, thing, concept_id, conceptscheme_id, new_type, uri):
        self.delete_thing(thing)
        self.session.flush()
        thing = Concept() if new_type == 'concept' else Collection()
        thing.type = new_type
        thing.concept_id = concept_id
        thing.conceptscheme_id = conceptscheme_id
        thing.uri = uri
        self.save(thing)
        return thing

    def delete_thing(self, thing):
        """

        :param thing: the thing to delete
        """
        self.session.delete(thing)

    def get_by_list_type(self, list_type):
        """

        :param list_type: a specific list type
        :return: all results for the specific list type
        """
        return self.session.execute(select(list_type)).scalars().all()

    def get_match_type(self, match_type):
        return self.session.execute(
            select(MatchType).filter(MatchType.name == match_type)
        ).scalar_one()

    def get_match(self, uri, matchtype_id, concept_id):
        return self.session.execute(
            select(Match).filter(
                Match.uri == uri,
                Match.matchtype_id == matchtype_id,
                Match.concept_id == concept_id,
            )
        ).scalar_one()

    def get_all_label_types(self):
        return self.session.execute(select(LabelType)).scalars().all()

    def get_next_cid(self, conceptscheme_id, id_generation_strategy):
        if id_generation_strategy == IDGenerationStrategy.NUMERIC:
            max_id = self.session.execute(
                select(func.max(sa.cast(Thing.concept_id, sa.Integer))).filter_by(
                    conceptscheme_id=conceptscheme_id
                )
            ).scalar_one()
            return max_id + 1 if max_id else 1
        elif id_generation_strategy == IDGenerationStrategy.GUID:
            return str(uuid.uuid4())
        else:
            raise ValueError('unsupported id_generation_strategy')

    def get_hierarchy_ids(
        self, conceptscheme_id, start_ids, concept_type, property_list_name
    ):
        """
        Get all concept_ids reachable by following a relationship recursively.

        Uses a recursive CTE query to efficiently traverse the hierarchy in a
        single database round-trip, instead of issuing one query per node.

        :param conceptscheme_id: The conceptscheme to search within.
        :param list start_ids: Starting concept_ids to traverse from.
        :param str concept_type: Only follow nodes of this type
            (``'concept'`` or ``'collection'``), or ``None`` for any type.
        :param str property_list_name: The relationship to follow
            (``'narrower_concepts'``, ``'broader_concepts'``, ``'members'``,
            ``'member_of'``).
        :rtype: set
        :returns: A set of all reachable concept_ids (strings).
        """
        if not start_ids:
            return set()

        # Resolve the starting concept_ids to internal surrogate ids,
        # filtering by conceptscheme and optionally by type.
        start_query = select(Thing.id, Thing.concept_id).where(
            Thing.concept_id.in_(start_ids),
            Thing.conceptscheme_id == conceptscheme_id,
        )
        if concept_type is not None:
            start_query = start_query.where(Thing.type == concept_type)

        start_rows = self.session.execute(start_query).all()
        if not start_rows:
            return set()

        start_internal_ids = [row[0] for row in start_rows]
        result_concept_ids = {row[1] for row in start_rows}

        # Build the recursive CTE based on the relationship type.
        hierarchy_cte = self._build_hierarchy_cte(
            start_internal_ids, concept_type, property_list_name
        )

        # Execute the CTE and collect all reachable concept_ids.
        query = select(Thing.concept_id).join(
            hierarchy_cte, Thing.id == hierarchy_cte.c.id
        )
        rows = self.session.execute(query).all()
        result_concept_ids |= {row[0] for row in rows}
        return result_concept_ids

    def _build_hierarchy_cte(
        self, start_internal_ids, concept_type, property_list_name
    ):
        """
        Build a recursive CTE that traverses a hierarchy relationship.

        :param list start_internal_ids: Internal surrogate ids to start from.
        :param str concept_type: Type filter or ``None``.
        :param str property_list_name: The relationship to follow.
        :returns: A CTE query yielding all reachable internal ids.
        """
        # Determine the association table and join columns based on the
        # relationship being traversed.
        if property_list_name == 'narrower_concepts':
            assoc_table = concept_hierarchy_concept
            from_col = assoc_table.c.concept_id_broader
            to_col = assoc_table.c.concept_id_narrower
        elif property_list_name == 'broader_concepts':
            assoc_table = concept_hierarchy_concept
            from_col = assoc_table.c.concept_id_narrower
            to_col = assoc_table.c.concept_id_broader
        elif property_list_name == 'members':
            assoc_table = collection_concept
            from_col = assoc_table.c.collection_id
            to_col = assoc_table.c.concept_id
        elif property_list_name == 'member_of':
            assoc_table = collection_concept
            from_col = assoc_table.c.concept_id
            to_col = assoc_table.c.collection_id
        else:
            raise ValueError(f'Unsupported property_list_name: {property_list_name}')

        # Base case: direct children of the starting nodes.
        base = select(
            to_col.label('id'),
        ).where(
            from_col.in_(start_internal_ids),
        )

        # Apply type filter by joining with the concept table.
        if concept_type is not None:
            base = base.join(Thing, Thing.id == to_col).where(
                Thing.type == concept_type,
            )

        hierarchy = base.cte(name='hierarchy', recursive=True)

        # Recursive case: follow the relationship from results found so far.
        recursive = select(
            to_col.label('id'),
        ).where(
            from_col == hierarchy.c.id,
        )

        if concept_type is not None:
            recursive = recursive.join(Thing, Thing.id == to_col).where(
                Thing.type == concept_type,
            )

        hierarchy = hierarchy.union(recursive)
        return hierarchy


class LanguagesManager(DataManager):
    """
    A :class:`DataManager` for
    :class:`Languages <skosprovider_sqlalchemy.models.Language>.`
    """

    def __init__(self, session):
        super().__init__(session)

    def get(self, language_id):
        return self.session.execute(
            select(Language).filter(Language.id == language_id)
        ).scalar_one()

    def save(self, language):
        """

        :param language: language to save
        :return: saved language
        """
        self.session.add(language)
        self.session.flush()
        return language

    def delete(self, language):
        """

        :param language: the language to delete
        """
        self.session.delete(language)

    def get_all(self):
        """

        :return: list of all languages
        """
        return self.session.execute(select(Language)).scalars().all()

    def get_all_sorted(self, sort_coll, sort_desc):
        """

        :param sort_coll: sort on this column
        :param sort_desc: descending or not
        :return: sorted list of languages
        """
        if sort_desc:
            return (
                self.session.execute(select(Language).order_by(desc(sort_coll)))
                .scalars()
                .all()
            )
        else:
            return (
                self.session.execute(select(Language).order_by(sort_coll))
                .scalars()
                .all()
            )

    def count_languages(self, language_tag):
        return self.session.execute(
            select(func.count(Language.id)).filter(Language.id == language_tag)
        ).scalar_one()


class AuditManager(DataManager):
    """
    A data manager for logging the visit.
    """

    def save(self, visit_log):
        """
        save a certain visit
        :param visit_log: log of visit to save
        :return: The saved visit log
        """
        self.session.add(visit_log)
        self.session.flush()
        return visit_log

    @popular_concepts.cache_on_arguments(expiration_time=86400)
    def get_most_popular_concepts_for_conceptscheme(
        self, conceptscheme_id, max_results=5, period='last_month'
    ):
        """
        get the most popular concepts for a conceptscheme
        :param conceptscheme_id: id of the conceptscheme
        :param max_results: maximum number of results, default 5
        :param period: 'last_day' or 'last_week' or 'last_month' or 'last_year', default 'last_mont h'
        :return: List of the most popular concepts of a conceptscheme over a certain period
        """

        start_date = self._get_first_day(period)
        rows = self.session.execute(
            select(
                ConceptVisitLog.concept_id,
                func.count(ConceptVisitLog.concept_id).label('count'),
            )
            .filter(
                ConceptVisitLog.conceptscheme_id == str(conceptscheme_id),
                ConceptVisitLog.visited_at >= start_date,
            )
            .group_by(ConceptVisitLog.concept_id)
            .order_by(desc('count'))
            .limit(max_results)
        ).all()
        results = []
        for row in rows:
            results.append(
                {'concept_id': row.concept_id, 'scheme_id': conceptscheme_id}
            )
        return results

    @staticmethod
    def _get_first_day(period):
        """
        get the first day of a certain period until now
        :param period: 'last_day' or 'last_week' or 'last_month' or 'last_year'
        :return: (string) the first day of the period
        """
        d = date.today()
        datetime.combine(d, datetime.min.time())
        start_date = d - dateutil.relativedelta.relativedelta(
            days=1 if period == 'last_day' else 0,
            weeks=1 if period == 'last_week' else 0,
            months=1 if period == 'last_month' else 0,
            years=1 if period == 'last_year' else 0,
        )
        return start_date.strftime('%Y-%m-%d')


class CountsManager(DataManager):
    """
    A data manager that deals with triple counts.
    """

    def save(self, counts):
        """
        Save a certain counts object

        :param atramhasis.data.models.ConceptschemeCounts counts: Counts object to save

        :return: The saved count
        """
        self.session.add(counts)
        self.session.flush()
        return counts

    def get_most_recent_count_for_scheme(self, conceptscheme_id):
        recent = self.session.execute(
            select(ConceptschemeCounts)
            .filter(ConceptschemeCounts.conceptscheme_id == conceptscheme_id)
            .order_by(desc('counted_at'))
        ).scalar_one()
        return recent


class ProviderDataManager(DataManager):
    """A data manager for managing Providers."""

    def get_provider_by_id(self, provider_id) -> Provider:
        return self.session.execute(
            select(Provider).filter(Provider.id == provider_id)
        ).scalar_one()

    def get_all_providers(self) -> List[Provider]:
        """
        Retrieve all providers from the database.

        :return: All providers
        """
        return self.session.execute(select(Provider)).scalars().all()



import os

from atramhasis.data.models import Base
from pyramid.config import Configurator
from pyramid.session import SignedCookieSessionFactory
from pyramid.settings import aslist
from sqlalchemy import engine_from_config


def main(global_config, **settings):
    """This function returns a Pyramid WSGI application."""

    # update settings
    settings["layout.focus_conceptschemes"] = aslist(
        settings["layout.focus_conceptschemes"], flatten=False
    )

    # set up dump location
    dump_location = settings["atramhasis.dump_location"]
    if not os.path.exists(dump_location):
        os.makedirs(dump_location)

    with Configurator(settings=settings) as config:
        # set default session factory
        atramhasis_session_factory = SignedCookieSessionFactory(
            settings["atramhasis.session_factory.secret"]
        )
        config.set_session_factory(atramhasis_session_factory)

        # Set up atramhasis
        config.include("atramhasis")
        # Set up atramhasis db
        config.include("atramhasis:data.db")

        config.scan()
        return config.make_wsgi_app()



import os

from pyramid.renderers import render
from pyramid.response import FileResponse
from pyramid.response import Response
from pyramid.view import view_config
from pyramid.view import view_defaults
from pyramid_skosprovider.views import ProviderView
from skosprovider_rdf import utils

from atramhasis.audit import audit
from atramhasis.errors import ConceptNotFoundException
from atramhasis.errors import ConceptSchemeNotFoundException
from atramhasis.errors import SkosRegistryNotFoundException
from atramhasis.rdf import void_dumper


@view_defaults()
class AtramhasisVoid:

    def __init__(self, request):
        self.request = request
        if hasattr(request, 'skos_registry') and request.skos_registry is not None:
            self.skos_registry = self.request.skos_registry
        else:
            raise SkosRegistryNotFoundException()   # pragma: no cover

    @view_config(route_name='atramhasis.rdf_void_turtle_ext')
    def rdf_void_turtle(self):
        graph = void_dumper(self.request, self.skos_registry)
        response = Response(content_type='text/turtle')
        response.text = graph.serialize(format='turtle')
        response.content_disposition = 'attachment; filename="void.ttl"'
        return response


@view_defaults()
class AtramhasisRDF:

    def __init__(self, request):
        self.request = request
        self.scheme_id = self.request.matchdict['scheme_id']
        if hasattr(request, 'skos_registry') and request.skos_registry is not None:
            self.skos_registry = self.request.skos_registry
        else:
            raise SkosRegistryNotFoundException()   # pragma: no cover
        self.provider = self.skos_registry.get_provider(self.scheme_id)
        if not self.provider:
            raise ConceptSchemeNotFoundException(self.scheme_id)   # pragma: no cover
        if 'c_id' in self.request.matchdict.keys():
            self.c_id = self.request.matchdict['c_id']
            if not self.provider.get_by_id(self.c_id):
                raise ConceptNotFoundException(self.c_id)

    @audit
    @view_config(route_name='skosprovider.conceptscheme.cs', accept='application/rdf+xml')
    @view_config(route_name='skosprovider.conceptscheme.cs.rdf')
    def rdf_full_export(self):
        dump_location = self.request.registry.settings['atramhasis.dump_location']
        filename = os.path.join(dump_location, '%s-full.rdf' % self.scheme_id)
        return FileResponse(
            filename,
            request=self.request,
            content_type='application/rdf+xml',
            cache_max_age=86400
        )

    @audit
    @view_config(route_name='skosprovider.conceptscheme.cs', accept='text/turtle')
    @view_config(route_name='skosprovider.conceptscheme.cs', accept='application/x-turtle')
    @view_config(route_name='skosprovider.conceptscheme.cs.ttl')
    def rdf_full_export_turtle(self):
        dump_location = self.request.registry.settings['atramhasis.dump_location']
        filename = os.path.join(dump_location, '%s-full.ttl' % self.scheme_id)
        return FileResponse(
            filename,
            request=self.request,
            content_type='text/turtle',
            cache_max_age=86400
        )

    @audit
    @view_config(route_name='skosprovider.conceptscheme', accept='application/rdf+xml')
    @view_config(route_name='skosprovider.conceptscheme.rdf')
    def rdf_conceptscheme_export(self):
        graph = utils.rdf_conceptscheme_dumper(self.provider)
        response = Response(content_type='application/rdf+xml')
        response.text = graph.serialize(format='xml')
        response.content_disposition = 'attachment; filename="{}.rdf"'.format(str(self.scheme_id))
        return response

    @view_config(route_name='skosprovider.conceptscheme', accept='text/turtle')
    @view_config(route_name='skosprovider.conceptscheme', accept='application/x-turtle')
    @view_config(route_name='skosprovider.conceptscheme.ttl')
    def rdf_conceptscheme_export_turtle(self):
        graph = utils.rdf_conceptscheme_dumper(self.provider)
        response = Response(content_type='text/turtle')
        response.text = graph.serialize(format='turtle')
        response.content_disposition = 'attachment; filename="{}.ttl"'.format(str(self.scheme_id))
        return response

    @audit
    @view_config(route_name='skosprovider.c', accept='application/rdf+xml')
    @view_config(route_name='skosprovider.c.rdf')
    def rdf_individual_export(self):
        graph = utils.rdf_c_dumper(self.provider, self.c_id)
        response = Response(content_type='application/rdf+xml')
        response.text = graph.serialize(format='xml')
        response.content_disposition = 'attachment; filename="{}.rdf"'.format(str(self.c_id))
        return response

    @audit
    @view_config(route_name='skosprovider.c', accept='text/turtle')
    @view_config(route_name='skosprovider.c', accept='application/x-turtle')
    @view_config(route_name='skosprovider.c.ttl')
    def rdf_individual_export_turtle(self):
        graph = utils.rdf_c_dumper(self.provider, self.c_id)
        response = Response(content_type='text/turtle')
        response.text = graph.serialize(format='turtle')
        response.content_disposition = 'attachment; filename="{}.ttl"'.format(str(self.c_id))
        return response

    @audit
    @view_config(route_name='skosprovider.conceptscheme', permission='view', accept='application/ld+json')
    @view_config(route_name='skosprovider.conceptscheme.jsonld', permission='view')
    def get_conceptscheme_jsonld(self):
        conceptscheme = ProviderView(self.request).get_conceptscheme_jsonld()
        response = Response(content_type='application/ld+json')
        response.text = render('skosjsonld', conceptscheme, self.request)
        response.content_disposition = 'attachment; filename="{}.jsonld"'.format(str(self.scheme_id))
        return response

    @audit
    @view_config(route_name='skosprovider.c', permission='view', accept='application/ld+json')
    @view_config(route_name='skosprovider.c.jsonld', permission='view')
    def get_concept(self):
        concept = ProviderView(self.request).get_concept()
        response = Response(content_type='application/ld+json')
        response.text = render('skosjsonld', concept, self.request)
        response.content_disposition = 'attachment; filename="{}.jsonld"'.format(str(self.c_id))
        return response



import enum

from skosprovider_sqlalchemy.models import ConceptScheme
from sqlalchemy import Column
from sqlalchemy import DateTime
from sqlalchemy import Enum
from sqlalchemy import ForeignKey
from sqlalchemy import Integer
from sqlalchemy import JSON
from sqlalchemy import String
from sqlalchemy import Text
from sqlalchemy.orm import declarative_base
from sqlalchemy.ext.hybrid import hybrid_property
from sqlalchemy.orm import relationship
from sqlalchemy.sql import func

Base = declarative_base()


class IDGenerationStrategy(enum.Enum):
    NUMERIC = enum.auto()
    GUID = enum.auto()
    MANUAL = enum.auto()


class ExpandStrategy(enum.Enum):
    RECURSE = 'recurse'
    VISIT = 'visit'


class ConceptschemeVisitLog(Base):
    __tablename__ = 'conceptscheme_visit_log'
    id = Column(Integer, primary_key=True, autoincrement=True)
    conceptscheme_id = Column(String(25), nullable=False)
    visited_at = Column(DateTime, default=func.now(), nullable=False)
    origin = Column(String(25), nullable=False)


class ConceptVisitLog(Base):
    __tablename__ = 'concept_visit_log'
    id = Column(Integer, primary_key=True, autoincrement=True)
    concept_id = Column(String, nullable=False)
    conceptscheme_id = Column(String(25), nullable=False)
    visited_at = Column(DateTime, default=func.now(), nullable=False)
    origin = Column(String(25), nullable=False)


class ConceptschemeCounts(Base):
    __tablename__ = 'conceptscheme_counts'
    id = Column(Integer, primary_key=True, autoincrement=True)
    conceptscheme_id = Column(String(25), nullable=False)
    counted_at = Column(DateTime, default=func.now(), nullable=False)
    triples = Column(Integer, nullable=False)
    conceptscheme_triples = Column(Integer, nullable=False)
    avg_concept_triples = Column(Integer, nullable=False)


class Provider(Base):
    __tablename__ = 'provider'

    id = Column(String, primary_key=True)
    conceptscheme_id = Column(
        Integer,
        ForeignKey(ConceptScheme.id),
        nullable=False,
    )
    uri_pattern = Column(Text, nullable=False)
    meta = Column('metadata', JSON, nullable=False)  # metadata is reserved in sqlalchemy
    expand_strategy = Column(Enum(ExpandStrategy))

    conceptscheme = relationship(
        ConceptScheme, uselist=False, single_parent=True, cascade='all, delete-orphan',
    )

    @hybrid_property
    def default_language(self):
        return self.meta.get('default_language')

    @default_language.setter
    def default_language(self, value):
        self.meta['default_language'] = value

    @hybrid_property
    def force_display_language(self):
        return self.meta.get('atramhasis.force_display_language')

    @force_display_language.setter
    def force_display_language(self, value):
        self.meta['atramhasis.force_display_language'] = value

    @hybrid_property
    def id_generation_strategy(self):
        return IDGenerationStrategy[self.meta.get('atramhasis.id_generation_strategy')]

    @id_generation_strategy.setter
    def id_generation_strategy(self, value):
        self.meta['atramhasis.id_generation_strategy'] = value.name

    @hybrid_property
    def subject(self):
        return self.meta.get('subject')

    @subject.setter
    def subject(self, value):
        self.meta['subject'] = value



import os
import tempfile

from pyramid.paster import get_appsettings

from atramhasis import main
from tests import DbTest
from tests import setup_db


here = os.path.dirname(__file__)
settings = get_appsettings(os.path.join(here, '../', 'tests/conf_test.ini'))


def setUpModule():
    setup_db()


class TestConfig(DbTest):
    def test_config(self):
        app = main({}, **settings)
        self.assertIsNotNone(app)

    def test_config_alt_dump_location(self):
        with tempfile.TemporaryDirectory() as temp_dir:
            settings['atramhasis.dump_location'] = temp_dir
            app = main({}, **settings)
            self.assertIsNotNone(app)
